"use strict";

/* exported logger */
var Promise = require("bluebird");

var AbstractGuiElement = require('../AbstractGuiElement');
var GuiConnector = require('../../GuiConnector');

var Functions = require('../../functions');
var logger = require('../../logger');

var guiUtils = new (require('../leftPanel/GuiUtils'))();

function AddProjectDialog(params) {
  AbstractGuiElement.call(this, params);
  var self = this;
  self.registerListenerType("onFileUpload");
  $(self.getElement()).addClass("minerva-edit-project-dialog");

  self.createGui();
}

AddProjectDialog.prototype = Object.create(AbstractGuiElement.prototype);
AddProjectDialog.prototype.constructor = AddProjectDialog;

AddProjectDialog.prototype.createGui = function() {
  var self = this;
  var element = self.getElement();

  var tabDiv = Functions.createElement({
    type : "div",
    name : "tabView",
    className : "tabbable boxed parentTabs"
  });
  element.appendChild(tabDiv);

  var tabMenuDiv = Functions.createElement({
    type : "ul",
    className : "nav nav-tabs"
  });
  tabDiv.appendChild(tabMenuDiv);

  var tabContentDiv = Functions.createElement({
    type : "div",
    className : "tab-content"
  });
  tabDiv.appendChild(tabContentDiv);

  self.createGeneralTab(tabMenuDiv, tabContentDiv);
  self.createOverlaysTab(tabMenuDiv, tabContentDiv);
  self.createSubmapsTab(tabMenuDiv, tabContentDiv);
  self.createOverviewImagesTab(tabMenuDiv, tabContentDiv);
};

AddProjectDialog.prototype.createGeneralTab = function(tabMenuDiv, tabContentDiv) {
  var self = this;
  self.addTab({
    tabMenuDiv : tabMenuDiv,
    tabContentDiv : tabContentDiv,
    name : "GENERAL",
    id : "add_project_general_tab",
    content : self.createGeneralTabContent(),
  });

};

AddProjectDialog.prototype.addTab = function(params) {
  var navLi = guiUtils.createTabMenuObject({
    id : params.id,
    name : params.name,
    navigationBar : params.tabMenuDiv
  });
  params.tabMenuDiv.appendChild(navLi);

  var contentDiv = guiUtils.createTabContentObject({
    id : params.id,
    navigationObject : navLi,
    navigationBar : params.tabMenuDiv
  });

  if (params.content !== undefined) {
    contentDiv.appendChild(params.content);
  }

  params.tabContentDiv.appendChild(contentDiv);
};

AddProjectDialog.prototype.createGeneralTabContent = function() {
  var self = this;

  var result = new Functions.createElement({
    type : "div",
  });

  var table = new Functions.createElement({
    type : "div",
    style : "display:table"
  });
  result.appendChild(table);

  var fileInput = Functions.createElement({
    type : "input",
    inputType : "file",
    name : "project-file",
  });
  fileInput.addEventListener("change", function() {
    return self.callListeners("onFileUpload", fileInput.files[0]);
  }, false);
  self.addListener("onFileUpload", function(e) {
    var file = e.arg;
    return self.processFile(file);
  });
  self.addListener("onFileUpload", function(e) {
    var file = e.arg;
    if (file.name.lastIndexOf('.') > 0) {
      return self.setProjectId(file.name.substring(0, file.name.lastIndexOf('.')));
    } else {
      return file.name;
    }
  });
  self.addListener("onFileUpload", function(e) {
    var file = e.arg;
    return self.setFileParserForFilename(file.name);
  });
  table.appendChild(self.createRow(guiUtils.createLabel("Upload file: "), fileInput));

  var fileFormatSelect = Functions.createElement({
    type : "select",
    name : "project-format",
  });

  table.appendChild(self.createRow(guiUtils.createLabel("File format: "), fileFormatSelect));

  table.appendChild(self.createInputRow("ProjectId:", "id", "project-id"));
  table.appendChild(self.createInputRow("Project name:", "UNKNOWN DISEASE MAP", "project-name"));
  table.appendChild(self.createInputRow("Project Disease:", "", "project-disease"));
  table.appendChild(self.createInputRow("Organism:", "", "project-organism"));
  table.appendChild(self.createInputRow("Version:", "", "project-version"));

  table.appendChild(self.createCheckboxRow("Annotate model automatically:", false, "project-annotate-automatically"));
  table.appendChild(self.createCheckboxRow("Verify manual annotations:", false, "project-verify-annotations"));
  table.appendChild(self.createCheckboxRow("Cache data:", false, "project-cache-data"));
  table.appendChild(self.createCheckboxRow("Auto margin:", true, "project-auto-margin"));
  table.appendChild(self.createCheckboxRow("Display as SBGN:", false, "project-sbgn-visualization"));
  table.appendChild(self.createCheckboxRow("Semantic zooming:", false, "project-semantic-zooming"));

  var saveProjectButton = Functions.createElement({
    type : "button",
    name : "saveProject",
    content : '<span class="ui-icon ui-icon-disk"></span>&nbsp;SAVE',
    onclick : function() {
      return self.onSaveClicked().then(function() {
        return self.close();
      }, GuiConnector.alert);
    },
  });
  var cancelButton = Functions.createElement({
    type : "button",
    name : "cancelProject",
    content : '<span class="ui-icon ui-icon-cancel"></span>&nbsp;CANCEL',
    onclick : function() {
      return self.close();
    },
  });
  var menuRow = Functions.createElement({
    type : "div",
    className : "minerva-menu-row",
    style : "display:table-row; margin:10px",
  });
  result.appendChild(menuRow);
  menuRow.appendChild(saveProjectButton);
  menuRow.appendChild(cancelButton);

  return result;
};
AddProjectDialog.prototype.createInputRow = function(labelName, defaultValue, inputName) {
  var label = new Functions.createElement({
    type : "div",
    style : "display:table-cell",
    content : labelName,
  });
  var input = new Functions.createElement({
    type : "div",
    style : "display:table-cell",
    content : "<input name='" + inputName + "' value='" + defaultValue + "'/>",
  });
  return this.createRow(label, input);
};

AddProjectDialog.prototype.createCheckboxRow = function(labelName, defaultValue, inputName) {
  var label = new Functions.createElement({
    type : "div",
    style : "display:table-cell",
    content : labelName,
  });
  var checked = "";
  if (defaultValue) {
    checked = "checked";
  }
  var checkbox = new Functions.createElement({
    type : "div",
    style : "display:table-cell",
    content : "<input type='checkbox' name='" + inputName + "' " + checked + "/>",
  });
  return this.createRow(label, checkbox);
};

AddProjectDialog.prototype.createRow = function(label, value) {
  var result = new Functions.createElement({
    type : "div",
    style : "display:table-row"
  });
  if (label.tagName.toLowerCase() !== 'div') {
    var labelContainer = new Functions.createElement({
      type : "div",
      style : "display:table-cell",
    });
    labelContainer.appendChild(label);
    label = labelContainer;
  }
  result.appendChild(label);
  if (value.tagName.toLowerCase() !== 'div') {
    var valueContainer = new Functions.createElement({
      type : "div",
      style : "display:table-cell",
    });
    valueContainer.appendChild(value);
    value = valueContainer;
  }
  result.appendChild(value);
  return result;

};
AddProjectDialog.prototype.createOverlaysTab = function(tabMenuDiv, tabContentDiv) {
  var self = this;
  self.addTab({
    tabMenuDiv : tabMenuDiv,
    tabContentDiv : tabContentDiv,
    name : "OVERLAYS",
    id : "project_overlays_tab",
    content : self.createOverlaysTabContent(),
  });
};

AddProjectDialog.prototype.createOverlaysTabContent = function() {
  var self = this;
  var result = Functions.createElement({
    type : "div",
  });
  result.appendChild(self._createOverlayTable());
  return result;
};

AddProjectDialog.prototype._createOverlayTable = function() {
  var result = Functions.createElement({
    type : "div",
    style : "margin-top:10px;",
  });

  return result;
};

AddProjectDialog.prototype.createSubmapsTab = function(tabMenuDiv, tabContentDiv) {
  var self = this;
  self.addTab({
    tabMenuDiv : tabMenuDiv,
    tabContentDiv : tabContentDiv,
    name : "USERS",
    id : "project_submaps_tab",
    content : self.createSubmapsTabContent(),
  });
};

AddProjectDialog.prototype.createSubmapsTabContent = function() {
  var result = Functions.createElement({
    type : "div",
    style : "margin-top:10px;",
  });
  return result;
};

AddProjectDialog.prototype.createOverviewImagesTab = function(tabMenuDiv, tabContentDiv) {
  var self = this;
  self.addTab({
    tabMenuDiv : tabMenuDiv,
    tabContentDiv : tabContentDiv,
    name : "USERS",
    id : "project_overview_images_tab",
    content : self.createOverviewImagesTabContent(),
  });
};

AddProjectDialog.prototype.createOverviewImagesTabContent = function() {
  var result = Functions.createElement({
    type : "div",
    style : "margin-top:10px;",
  });
  return result;
};

AddProjectDialog.prototype.init = function() {
  var self = this;
  return ServerConnector.getConfiguration().then(function(configuration) {
    var select = $("[name='project-format']", self.getElement())[0];
    var option = Functions.createElement({
      type : "option",
      content : "---",
    });
    option.selected = true;
    select.appendChild(option);
    var converters = configuration.getModelConverters();
    for (var i = 0; i < converters.length; i++) {
      var converter = converters[i];
      option = Functions.createElement({
        type : "option",
        content : converter.name,
      });
      option.data = converter.extension;
      option.value = converter.handler;
      select.appendChild(option);
    }
  });
};

AddProjectDialog.prototype.destroy = function() {
  $(this.getElement()).dialog("destroy");
};

AddProjectDialog.prototype.open = function() {
  var self = this;
  var div = self.getElement();
  if (!$(div).hasClass("ui-dialog-content")) {
    $(div).dialog({
      title : "ADD PROJECT",
      width : window.innerWidth / 2,
      height : window.innerHeight / 2,
    });
  }
  $(div).dialog("open");
};

AddProjectDialog.prototype.close = function() {
  var self = this;
  $(self.getElement()).dialog("close");
};

AddProjectDialog.prototype.clear = function() {
  var self = this;
  $("input", self.getElement()).val("");
  self.setFileContent(undefined);
  return Promise.resolve();
};

AddProjectDialog.prototype.processFile = function(file) {
  var self = this;
  self.setFileContent(null);
  if (file) {
    return new Promise(function(resolve, reject) {
      var reader = new FileReader();
      reader.readAsText(file, "UTF-8");
      reader.onload = function(evt) {
        try {
          self.setFileContent(evt.target.result);
          resolve(self.getFileContent());
        } catch (error) {
          reject(error);
        }
      };
      reader.onerror = function() {
        reject(new Error("Problem reading file"));
      };
    });
  } else {
    return Promise.resolve(null);
  }
};

AddProjectDialog.prototype.setFileContent = function(fileContent) {
  this._fileContent = fileContent;
};
AddProjectDialog.prototype.getFileContent = function() {
  return this._fileContent;
};

AddProjectDialog.prototype.setProjectId = function(projectId) {
  $("[name='project-id']", this.getElement()).val(projectId);
};

AddProjectDialog.prototype.getProjectId = function() {
  return $("[name='project-id']", this.getElement()).val();
};

AddProjectDialog.prototype.setFileParserForFilename = function(filename) {
  var self = this;
  var select = $("[name='project-format']", self.getElement)[0];
  var options = select.options;
  var optionId = 0;
  for (var i = 0; i < options.length; i++) {
    var option = options[i];
    if (option.value !== undefined) {
      var extension = option.data;
      if (filename.endsWith(extension) && optionId <= 0) {
        optionId = i;
      }
    }
  }
  options[optionId].selected = true;
};
AddProjectDialog.prototype.getConverter = function() {
  var self = this;
  var select = $("[name='project-format']", self.getElement)[0];
  return ServerConnector.getConfiguration().then(function(configuration) {
    var options = select.options;
    var handler = select.options[select.selectedIndex].value;
    var converters = configuration.getModelConverters();
    for (var i = 0; i < converters.length; i++) {
      var converter = converters[i];
      if (handler === converter.handler) {
        return converter;
      }
    }
    return null;
  });
};

module.exports = AddProjectDialog;
